@@ -222,13 +222,8 @@ clonePreparePaintScreen (CompScreen *s,
 	{
 	    cs->offset += msSinceLastPaint * 0.005f;
 	    if (cs->offset >= 1.0f)
-	    {
 		cs->offset = 1.0f;
-		cloneFinish (s);
-	    }
 	}
-
-	damageScreen (s);
     }
 
     UNWRAP (cs, s, preparePaintScreen);
@@ -275,8 +270,13 @@ cloneDonePaintScreen (CompScreen *s)
 {
     CLONE_SCREEN (s);
 
-    if (cs->grab && cs->offset)
+    if (cs->grab)
+    {
+	if (cs->offset == 1.0f)
+	    cloneFinish (s);
+
 	damageScreen (s);
+    }
 
     UNWRAP (cs, s, donePaintScreen);
     (*s->donePaintScreen) (s);
@@ -809,8 +809,13 @@ static void
 cloneFiniScreen (CompPlugin *p,
 		 CompScreen *s)
 {
+    int i;
+
     CLONE_SCREEN (s);
 
+    for (i = 0; i < cs->nClone; i++)
+	cloneRemove (s, i);
+
     UNWRAP (cs, s, preparePaintScreen);
     UNWRAP (cs, s, donePaintScreen);
     UNWRAP (cs, s, paintOutput);
