@@ -848,6 +848,12 @@ decorWindowUpdate (CompWindow *w,
 	if (w->state & CompWindowStateMaximizedVertMask)
 	    mask &= ~CWY;
 
+	if (w->saveMask & CWX)
+	    w->saveWc.x += moveDx;
+
+	if (w->saveMask & CWY)
+	    w->saveWc.y += moveDy;
+
 	if (mask)
 	    configureXWindow (w, mask, &xwc);
     }
@@ -1499,6 +1505,9 @@ decorInitScreen (CompPlugin *p,
 
     decorCheckForDmOnScreen (s, FALSE);
 
+    if (!ds->dmWin)
+	runCommand (s, dd->opt[DECOR_DISPLAY_OPTION_COMMAND].value.s);
+
     return TRUE;
 }
 
