@@ -161,7 +161,7 @@ paintOutputRegion (CompScreen	       *screen,
     static Region tmpRegion = NULL;
     CompWindow    *w;
     CompCursor	  *c;
-    int		  count, windowMask, backgroundMask, odMask;
+    int		  count, windowMask, backgroundMask, odMask, i;
     CompWindow	  *fullscreenWindow = NULL;
     CompWalker    walk;
     Bool          status;
@@ -246,19 +246,30 @@ paintOutputRegion (CompScreen	       *screen,
 		XSubtractRegion (tmpRegion, w->region, tmpRegion);
 
 	    /* unredirect top most fullscreen windows. */
-	    if (count == 0					      &&
-		!REGION_NOT_EMPTY (tmpRegion)			      &&
-		screen->opt[COMP_SCREEN_OPTION_UNREDIRECT_FS].value.b &&
-		XEqualRegion (w->region, &screen->region))
+	    if (count == 0 &&
+		screen->opt[COMP_SCREEN_OPTION_UNREDIRECT_FS].value.b)
 	    {
-		unredirectWindow (w);
-		fullscreenWindow = w;
+		if (XEqualRegion (w->region, &screen->region) &&
+		    !REGION_NOT_EMPTY (tmpRegion))
+		{
+		    fullscreenWindow = w;
+		}
+		else
+		{
+		    for (i = 0; i < screen->nOutputDev; i++)
+			if (XEqualRegion (w->region,
+					  &screen->outputDev[i].region))
+			    fullscreenWindow = w;
+		}
 	    }
 	}
 
 	count++;
     }
 
+    if (fullscreenWindow)
+	unredirectWindow (fullscreenWindow);
+
     (*screen->paintBackground) (screen, tmpRegion, backgroundMask);
 
     /* paint all windows from bottom to top */
@@ -462,6 +473,7 @@ paintOutput (CompScreen		     *screen,
     }						   \
     *(data)++ = (x1);				   \
     *(data)++ = (y2);				   \
+    *(data)++ = 0.0;				   \
     for (it = 0; it < n; it++)			   \
     {						   \
 	*(data)++ = COMP_TEX_COORD_X (&m[it], x2); \
@@ -469,6 +481,7 @@ paintOutput (CompScreen		     *screen,
     }						   \
     *(data)++ = (x2);				   \
     *(data)++ = (y2);				   \
+    *(data)++ = 0.0;				   \
     for (it = 0; it < n; it++)			   \
     {						   \
 	*(data)++ = COMP_TEX_COORD_X (&m[it], x2); \
@@ -476,13 +489,15 @@ paintOutput (CompScreen		     *screen,
     }						   \
     *(data)++ = (x2);				   \
     *(data)++ = (y1);				   \
+    *(data)++ = 0.0;				   \
     for (it = 0; it < n; it++)			   \
     {						   \
 	*(data)++ = COMP_TEX_COORD_X (&m[it], x1); \
 	*(data)++ = COMP_TEX_COORD_Y (&m[it], y1); \
     }						   \
     *(data)++ = (x1);				   \
-    *(data)++ = (y1)
+    *(data)++ = (y1);				   \
+    *(data)++ = 0.0
 
 #define ADD_QUAD(data, m, n, x1, y1, x2, y2)		\
     for (it = 0; it < n; it++)				\
@@ -492,6 +507,7 @@ paintOutput (CompScreen		     *screen,
     }							\
     *(data)++ = (x1);					\
     *(data)++ = (y2);					\
+    *(data)++ = 0.0;					\
     for (it = 0; it < n; it++)				\
     {							\
 	*(data)++ = COMP_TEX_COORD_XY (&m[it], x2, y2);	\
@@ -499,6 +515,7 @@ paintOutput (CompScreen		     *screen,
     }							\
     *(data)++ = (x2);					\
     *(data)++ = (y2);					\
+    *(data)++ = 0.0;					\
     for (it = 0; it < n; it++)				\
     {							\
 	*(data)++ = COMP_TEX_COORD_XY (&m[it], x2, y1);	\
@@ -506,13 +523,15 @@ paintOutput (CompScreen		     *screen,
     }							\
     *(data)++ = (x2);					\
     *(data)++ = (y1);					\
+    *(data)++ = 0.0;					\
     for (it = 0; it < n; it++)				\
     {							\
 	*(data)++ = COMP_TEX_COORD_XY (&m[it], x1, y1);	\
 	*(data)++ = COMP_TEX_COORD_YX (&m[it], x1, y1);	\
     }							\
     *(data)++ = (x1);					\
-    *(data)++ = (y1)
+    *(data)++ = (y1);					\
+    *(data)++ = 0.0;
 
 
 Bool
@@ -558,12 +577,12 @@ drawWindowGeometry (CompWindow *w)
 {
     int     texUnit = w->texUnits;
     int     currentTexUnit = 0;
-    int     stride = (1 + texUnit) * 2;
-    GLfloat *vertices = w->vertices + (stride - 2);
+    int     stride = w->vertexStride;
+    GLfloat *vertices = w->vertices + (stride - 3);
 
     stride *= sizeof (GLfloat);
 
-    glVertexPointer (2, GL_FLOAT, stride, vertices);
+    glVertexPointer (3, GL_FLOAT, stride, vertices);
 
     while (texUnit--)
     {
@@ -573,8 +592,8 @@ drawWindowGeometry (CompWindow *w)
 	    glEnableClientState (GL_TEXTURE_COORD_ARRAY);
 	    currentTexUnit = texUnit;
 	}
-	vertices -= 2;
-	glTexCoordPointer (2, GL_FLOAT, stride, vertices);
+	vertices -= w->texCoordSize;
+	glTexCoordPointer (w->texCoordSize, GL_FLOAT, stride, vertices);
     }
 
     glDrawArrays (GL_QUADS, 0, w->vCount);
@@ -638,7 +657,7 @@ addWindowGeometry (CompWindow *w,
 	pBox = region->rects;
 	nBox = region->numRects;
 
-	vSize = 2 + nMatrix * 2;
+	vSize = 3 + nMatrix * 2;
 
 	n = w->vCount / 4;
 
@@ -733,6 +752,8 @@ addWindowGeometry (CompWindow *w,
 	}
 
 	w->vCount	      = n * 4;
+	w->vertexStride       = vSize;
+	w->texCoordSize       = 2;
 	w->drawWindowGeometry = drawWindowGeometry;
     }
 }
